package hu.bme.mit.bookorder;

import java.util.HashMap;

import hu.bme.mit.bookorder.book.Book;
import hu.bme.mit.bookorder.handlers.BuyBookWorkItemHandler;
import hu.bme.mit.bookorder.handlers.CheckPriceInvocationWorkItemHandlerForDotNet;
import hu.bme.mit.bookorder.handlers.CheckPriceInvocationWorkItemHandlerForOSGi;

import org.drools.KnowledgeBase;
import org.drools.builder.KnowledgeBuilder;
import org.drools.builder.KnowledgeBuilderFactory;
import org.drools.builder.ResourceType;
import org.drools.io.ResourceFactory;
import org.drools.runtime.StatefulKnowledgeSession;
import org.drools.runtime.rule.FactHandle;

public class ProcessMain {
	public static void main(String[] args) throws Exception {
		KnowledgeBase kbase = readKnowledgeBase();
		StatefulKnowledgeSession ksession = kbase.newStatefulKnowledgeSession();
		
		final Book book = makeBook(); // !!
		HashMap<String, Object> context = new HashMap<String, Object>();
		context.put("book", book); 
		context.put("bookTitle", book.getTitle());
		
		FactHandle bookHandle = ksession.insert(book); // !!
		
		ksession.getWorkItemManager().registerWorkItemHandler("CheckPriceInvocationOSGi", new CheckPriceInvocationWorkItemHandlerForOSGi());
		ksession.getWorkItemManager().registerWorkItemHandler("CheckPriceInvocationDotNet", new CheckPriceInvocationWorkItemHandlerForDotNet());
		ksession.getWorkItemManager().registerWorkItemHandler("BuyBook", new BuyBookWorkItemHandler());

		ksession.startProcess("hu.bme.mit.wf.bookorder", context);
		ksession.fireAllRules();	// !!
	}

	private static Book makeBook() { // !!
		return new Book("1984", "George Orwell", "Secker and Warburg (London)", 1949, 326, Book.Genre.SCIENCE_FICTION);
	}

	private static KnowledgeBase readKnowledgeBase() throws Exception {
		KnowledgeBuilder kbuilder = KnowledgeBuilderFactory.newKnowledgeBuilder();
		kbuilder.add(ResourceFactory.newClassPathResource("bookorder.bpmn"), ResourceType.BPMN2);
        kbuilder.add(ResourceFactory.newClassPathResource("init.drl"), ResourceType.DRL);	// !!
        kbuilder.add(ResourceFactory.newClassPathResource("adjust.drl"), ResourceType.DRL); // !!
		return kbuilder.newKnowledgeBase();
	}
}
